__all__ = [
    '__title__', '__summary__', '__uri__', '__version__', '__author__',
    '__email__', '__license__', '__copyright__',
]

__title__ = 'Pil-Lite'
__summary__ = 'Python Imaging Library Lite'
__uri__ = 'https://github.com/alexa-infra/pil-lite'

__version__ = '0.1.3'

__author__ = 'Alexey Vasilyev'
__email__ = 'alexey@alexadotlife.com'

__license__ = 'MIT'
__copyright__ = 'Copyright 2014-2024 {0}'.format(__author__)
